import React, { useEffect, useState } from 'react';
import { getAddresses } from './service';
import './autocomplete.css';
import TextField from '@mui/material/TextField';
import Autocomplete from '@mui/material/Autocomplete';
import CircularProgress from '@mui/material/CircularProgress';

const allowedDisplayData = ['state', 'address', 'name', 'zip', 'city'];

const capitalizeWords = (text) => {
  return text
    .split(' ')
    .map((word) => word.charAt(0).toUpperCase() + word.slice(1))
    .join(' ');
};

const styleDisplayData = (type, value) => {
  switch (type) {
    case 'city':
    case 'address':
      return capitalizeWords(value);

    case 'state':
      return value.toUpperCase();

    default:
      return value;
  }
};

export const ReactBatchAutocomplete = ({
  placeholder = 'Enter your address',
  notFoundText = 'No results found',
  debounce = 700,
  apiKey,
  domainServer,
  handleError,
  suggestionList,
  onSelect,
  recentSearch = Array(),
  displayData = 'name',
  showNotFound = true,
  take = 5,
  filterType = [],
}) => {
  const [search, setSearch] = useState('');
  const [searchQuery, setSearchQuery] = useState('');
  const [open, setOpen] = useState(false);
  const [options, setOptions] = useState([]);
  const [recentSearchOption, setRecentSearchOption] = useState([]);
  const [loading, setLoading] = useState(false);
  const [renderRecentSearch, setRenderRecentSearch] = useState(true);
  const [allowApiCall, setAllowApiCall] = useState(false);

  useEffect(() => {
    search.length > 0 && setRenderRecentSearch(false);
    const timeout = setTimeout(() => {
      setSearchQuery(search);
    }, debounce);

    return () => {
      clearTimeout(timeout);
    };
  }, [search]);

  useEffect(() => {
    async function runProcesses() {
      if (
        searchQuery?.trim().length > 2 &&
        options?.filter((item) => item.name === searchQuery)?.length === 0
      ) {
        try {
          if (allowApiCall) {
            setLoading(true);
            let data = await getAddresses(
              apiKey,
              domainServer,
              searchQuery,
              take,
              filterType
            );
            if (data?.status?.code !== 200) {
              handleError?.(data?.status);
            } else {
              let batchAddress;
              let apiResponse = data?.result?.suggestions;

              batchAddress = apiResponse;
              setOptions(batchAddress);
              suggestionList?.(batchAddress);
              setOpen(true);
            }
            setLoading(false);
          }
        } catch (e) {
          handleError?.(e);
          setLoading(false);
        }
      }
    }
    runProcesses();
  }, [searchQuery, allowApiCall]);

  useEffect(() => {
    if (recentSearch?.length > 0) {
      setRecentSearchOption(recentSearch);
    }
  }, [recentSearch]);

  const renderDisplayData = (option) => {
    if (
      allowedDisplayData.includes(displayData.toLowerCase()) &&
      option[displayData]
    ) {
      return styleDisplayData(displayData, option[displayData]);
    }
    return option.name;
  };

  const handleChange = (data) => {
    setAllowApiCall(false);
    return !renderRecentSearch && onSelect?.(data);
  };

  return (
    <div className='autocomplete-wrapper'>
      <Autocomplete
        disablePortal
        className='custom-autocomplete'
        open={open}
        onOpen={() => {
          search.length > 2 && options.length > 0 && setOpen(true);
        }}
        groupBy={() => (renderRecentSearch ? 'Recent Searches' : '')}
        onClose={() => {
          setOpen(false);
        }}
        renderGroup={(value) =>
          renderRecentSearch ? (
            <div className='recent-wrapper' key={value.key}>
              <div className='recent-title'>
                <div className='recent-img' />
                <h6>{value.group}</h6>
              </div>
              {value.children}
            </div>
          ) : (
            value.children
          )
        }
        onInputChange={(event, value) => {
          setAllowApiCall(true);
          options.length === 0 && setOpen(false);
          if (!value && !recentSearch) {
            setOpen(false);
            setOptions([]);
          }
          setSearch((prevValue) => {
            if (
              prevValue?.length > value.length &&
              !value &&
              recentSearch?.length > 0
            ) {
              setRenderRecentSearch(true);
              setOpen(true);
              setOptions([]);
            }
            return value;
          });
        }}
        freeSolo={!showNotFound}
        getOptionLabel={(option) => renderDisplayData(option)}
        onChange={(event, data) => handleChange(data)}
        inputValue={search}
        options={renderRecentSearch ? recentSearchOption : options}
        noOptionsText={notFoundText}
        clearOnBlur={false}
        filterOptions={(options) => options}
        renderInput={(params) => (
          <TextField
            {...params}
            hiddenLabel
            placeholder={placeholder}
            id='filled-hidden-label-normal'
            onFocus={() =>
              recentSearch?.length > 0 && renderRecentSearch && setOpen(true)
            }
            inputProps={{ ...params.inputProps, maxLength: 25 }}
            InputProps={{
              ...params.InputProps,
              endAdornment: (
                <>
                  {loading ? (
                    <CircularProgress className='spinner' size={20} />
                  ) : null}
                  {params.InputProps.endAdornment}
                </>
              ),
            }}
          />
        )}
        renderOption={(props, option) => {
          return renderRecentSearch ? (
            <div {...props}>
              <div className='auto-wrapper-recent'>
                <div className='text-content'>{option.name}</div>
                <div className='img-wrap-recent' />
              </div>
            </div>
          ) : (
            <div {...props}>
              <div className='auto-wrapper'>
                <div className='img-wrap' />
                <div className='text-content'>
                  <strong>
                    {option?.name?.substr(0, search?.trim()?.length)}
                  </strong>
                  {option?.name?.substr(search?.trim()?.length)}
                </div>
              </div>
            </div>
          );
        }}
      />
    </div>
  );
};